@@ -1,6 +1,6 @@
 // -----------------------------------------------------------------------------------------------------
-// Copyright (c) 2006-2020, Knut Reinert & Freie Universität Berlin
-// Copyright (c) 2016-2020, Knut Reinert & MPI für molekulare Genetik
+// Copyright (c) 2006-2023, Knut Reinert & Freie Universität Berlin
+// Copyright (c) 2016-2023, Knut Reinert & MPI für molekulare Genetik
 // This file may be used, modified and/or redistributed under the terms of the 3-clause BSD-License
 // shipped with this file and also available at: https://github.com/seqan/seqan3/blob/master/LICENSE.md
 // -----------------------------------------------------------------------------------------------------
@@ -12,23 +12,24 @@
 
 #pragma once
 
-#include <regex>
+#include <algorithm>
+#include <concepts>
+#include <filesystem>
 #include <fstream>
+#include <ranges>
+#include <regex>
 #include <sstream>
 
 #include <seqan3/argument_parser/exceptions.hpp>
-#include <seqan3/core/concept/core_language.hpp>
-#include <seqan3/core/detail/to_string.hpp>
-#include <seqan3/core/type_list/traits.hpp>
-#include <seqan3/core/type_traits/basic.hpp>
+#include <seqan3/core/debug_stream/detail/to_string.hpp>
+#include <seqan3/core/debug_stream/range.hpp>
 #include <seqan3/io/detail/misc.hpp>
 #include <seqan3/io/detail/safe_filesystem_entry.hpp>
-#include <seqan3/range/container/concept.hpp>
-#include <seqan3/range/views/join.hpp>
-#include <seqan3/std/algorithm>
-#include <seqan3/std/concepts>
-#include <seqan3/std/filesystem>
-#include <seqan3/std/ranges>
+#include <seqan3/utility/concept.hpp>
+#include <seqan3/utility/type_list/traits.hpp>
+#include <seqan3/utility/type_pack/traits.hpp>
+#include <seqan3/utility/type_traits/basic.hpp>
+#include <seqan3/utility/views/join_with.hpp>
 
 namespace seqan3
 {
@@ -39,6 +40,18 @@ namespace seqan3
  *
  * \details
  *
+ * When adding (positional) options to the seqan3::argument_parser you may pass a
+ * [function object](https://en.cppreference.com/w/cpp/named_req/FunctionObject) that models
+ * seqan3::validator which checks the option value provided by the user for some constraint.
+ *
+ * SeqAn provides several common-use-case validators, e.g. the seqan3::arithmetic_range_validator.
+ *
+ * \include test/snippet/argument_parser/validators_2.cpp
+ *
+ * You can learn more about SeqAn validators in our tutorial.
+ *
+ * To implement your own validator please refer to the detailed concept description below.
+ *
  * ### Concepts and doxygen
  *
  * The requirements for this concept are given as related functions and type traits.
@@ -75,24 +88,30 @@ namespace seqan3
  * \details
  * \attention This is a concept requirement, not an actual function (however types satisfying this concept
  * will provide an implementation).
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 //!\}
 //!\cond
 template <typename validator_type>
-SEQAN3_CONCEPT validator = std::copyable<std::remove_cvref_t<validator_type>> &&
-                           requires(validator_type validator,
-                                    typename std::remove_reference_t<validator_type>::option_value_type value)
-{
-    typename std::remove_reference_t<validator_type>::option_value_type;
+concept validator =
+    std::copyable<std::remove_cvref_t<validator_type>>
+    && requires (validator_type validator, typename std::remove_reference_t<validator_type>::option_value_type value) {
+           typename std::remove_reference_t<validator_type>::option_value_type;
 
-    SEQAN3_RETURN_TYPE_CONSTRAINT(validator(value), std::same_as, void);
-    SEQAN3_RETURN_TYPE_CONSTRAINT(validator.get_help_page_message(), std::same_as, std::string);
-};
+           {
+               validator(value)
+               } -> std::same_as<void>;
+           {
+               validator.get_help_page_message()
+               } -> std::same_as<std::string>;
+       };
 //!\endcond
 
 /*!\brief A validator that checks whether a number is inside a given range.
  * \ingroup argument_parser
  * \implements seqan3::validator
+ * \tparam option_value_t The value type of the range; must model seqan3::arithmetic .
  *
  * \details
  *
@@ -101,19 +120,21 @@ SEQAN3_CONCEPT validator = std::copyable<std::remove_cvref_t<validator_type>> &&
  * exception whenever a given value does not lie inside the given min/max range.
  *
  * \include test/snippet/argument_parser/validators_1.cpp
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
+template <arithmetic option_value_t>
 class arithmetic_range_validator
 {
 public:
     //!\brief The type of value that this validator invoked upon.
-    using option_value_type = double;
+    using option_value_type = option_value_t;
 
     /*!\brief The constructor.
      * \param[in] min_ Minimum set for the range to test.
      * \param[in] max_ Maximum set for the range to test.
      */
-    arithmetic_range_validator(option_value_type const min_, option_value_type const max_) :
-        min{min_}, max{max_}
+    arithmetic_range_validator(option_value_type const min_, option_value_type const max_) : min{min_}, max{max_}
     {}
 
     /*!\brief Tests whether cmp lies inside [`min`, `max`].
@@ -133,12 +154,15 @@ public:
      * \throws seqan3::validation_error
      */
     template <std::ranges::forward_range range_type>
-    //!\cond
         requires arithmetic<std::ranges::range_value_t<range_type>>
-    //!\endcond
     void operator()(range_type const & range) const
     {
-        std::for_each(range.begin(), range.end(), [&] (auto cmp) { (*this)(cmp); });
+        std::for_each(range.begin(),
+                      range.end(),
+                      [&](auto cmp)
+                      {
+                          (*this)(cmp);
+                      });
     }
 
     //!\brief Returns a message that can be appended to the (positional) options help page info.
@@ -158,7 +182,7 @@ private:
 /*!\brief A validator that checks whether a value is inside a list of valid values.
  * \ingroup argument_parser
  * \implements seqan3::validator
- * \tparam option_value_t Must be a (container of) arithmetic type(s).
+ * \tparam option_value_t \copybrief seqan3::value_list_validator::option_value_type
  *
  * \details
  *
@@ -166,11 +190,13 @@ private:
  * The class than acts as a functor, that throws a seqan3::validation_error
  * exception whenever a given value is not in the given list.
  *
- * \note In order to simplify the chaining of validators, the option value type is deduced to `double` for ranges whose
- *       value type models seqan3::arithmetic, and to `std::string` if the ranges value type is convertible to it.
- *       Otherwise, the option value type is deduced to the value type of the range.
+ * \note In order to simplify the chaining of validators, the option value type is deduced to `std::string` if the
+ *       range's value type is convertible to it. Otherwise, the option value type is deduced to the value type of the
+ *       range.
  *
  * \include test/snippet/argument_parser/validators_2.cpp
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 template <typename option_value_t>
 class value_list_validator
@@ -195,13 +221,11 @@ public:
      * \param[in] rng The range of valid values to test.
      */
     template <std::ranges::forward_range range_type>
-    //!\cond
         requires std::constructible_from<option_value_type, std::ranges::range_rvalue_reference_t<range_type>>
-    //!\endcond
     value_list_validator(range_type rng)
     {
         values.clear();
-        std::ranges::move(std::move(rng), std::cpp20::back_inserter(values));
+        std::ranges::move(std::move(rng), std::back_inserter(values));
     }
 
     /*!\brief Constructing from a parameter pack.
@@ -209,11 +233,9 @@ public:
      *                      be constructible from each type in the parameter pack.
      * \param[in] opts The parameter pack values.
      */
-    template <typename ...option_types>
-    //!\cond
+    template <typename... option_types>
         requires ((std::constructible_from<option_value_type, option_types> && ...))
-    //!\endcond
-    value_list_validator(option_types && ...opts)
+    value_list_validator(option_types &&... opts)
     {
         (values.emplace_back(std::forward<option_types>(opts)), ...);
     }
@@ -235,12 +257,15 @@ public:
      * \throws seqan3::validation_error
      */
     template <std::ranges::forward_range range_type>
-    //!\cond
         requires std::convertible_to<std::ranges::range_value_t<range_type>, option_value_type>
-    //!\endcond
     void operator()(range_type const & range) const
     {
-        std::for_each(std::ranges::begin(range), std::ranges::end(range), [&] (auto cmp) { (*this)(cmp); });
+        std::for_each(std::ranges::begin(range),
+                      std::ranges::end(range),
+                      [&](auto cmp)
+                      {
+                          (*this)(cmp);
+                      });
     }
 
     //!\brief Returns a message that can be appended to the (positional) options help page info.
@@ -250,7 +275,6 @@ public:
     }
 
 private:
-
     //!\brief Minimum of the range to test.
     std::vector<option_value_type> values{};
 };
@@ -259,46 +283,25 @@ private:
  * \relates seqan3::value_list_validator
  * \{
  */
-//!\brief Deduction guide for a parameter pack over an arithmetic type.
-template <arithmetic ...option_types>
-//!\cond
-    requires (!(detail::is_char_adaptation_v<option_types> || ...))
-//!\endcond
-value_list_validator(option_types...) -> value_list_validator<double>;
-
-//!\brief Deduction guide for ranges over an arithmetic type.
-template <typename range_type>
-//!\cond
-    requires std::ranges::forward_range<std::decay_t<range_type>> &&
-             arithmetic<std::ranges::range_value_t<range_type>> &&
-             (!seqan3::detail::is_char_adaptation_v<std::ranges::range_value_t<range_type>>)
-//!\endcond
-value_list_validator(range_type && rng) -> value_list_validator<double>;
-
 //!\brief Given a parameter pack of types that are convertible to std::string, delegate to value type std::string.
-template <typename ...option_types>
-//!\cond
-    requires (std::constructible_from<std::string, std::decay_t<option_types>> && ...)
-//!\endcond
-value_list_validator(option_types...) -> value_list_validator<std::string>;
+template <typename option_type, typename... option_types>
+    requires (std::constructible_from<std::string, std::decay_t<option_types>> && ...
+              && std::constructible_from<std::string, std::decay_t<option_type>>)
+value_list_validator(option_type, option_types...) -> value_list_validator<std::string>;
 
 //!\brief Deduction guide for ranges over a value type convertible to std::string.
 template <typename range_type>
-//!\cond
-    requires (std::ranges::forward_range<std::decay_t<range_type>> &&
-              std::constructible_from<std::string, std::ranges::range_value_t<range_type>>)
-//!\endcond
+    requires (std::ranges::forward_range<std::decay_t<range_type>>
+              && std::constructible_from<std::string, std::ranges::range_value_t<range_type>>)
 value_list_validator(range_type && rng) -> value_list_validator<std::string>;
 
 //!\brief Deduction guide for a parameter pack.
-template <typename ...option_types>
-value_list_validator(option_types...) -> value_list_validator<seqan3::pack_traits::front<option_types...>>;
+template <typename option_type, typename... option_types>
+value_list_validator(option_type, option_types...) -> value_list_validator<option_type>;
 
 //!\brief Deduction guide for ranges.
 template <typename range_type>
-//!\cond
     requires (std::ranges::forward_range<std::decay_t<range_type>>)
-//!\endcond
 value_list_validator(range_type && rng) -> value_list_validator<std::ranges::range_value_t<range_type>>;
 //!\}
 
@@ -313,11 +316,12 @@ value_list_validator(range_type && rng) -> value_list_validator<std::ranges::ran
  *
  * The type can be further specialised for the seqan3::input_file_validator and the seqan3::output_file_validator
  * using the template argument to determine the valid extensions from the given file type.
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 class file_validator_base
 {
 public:
-
     //!\brief Type of values that are tested by validator.
     using option_value_type = std::string;
 
@@ -349,12 +353,16 @@ public:
      * \throws seqan3::validation_error
      */
     template <std::ranges::forward_range range_type>
-    //!\cond
-        requires std::convertible_to<std::ranges::range_value_t<range_type>, std::filesystem::path const &>
-    //!\endcond
+        requires (std::convertible_to<std::ranges::range_value_t<range_type>, std::filesystem::path const &>
+                  && !std::convertible_to<range_type, std::filesystem::path const &>)
     void operator()(range_type const & v) const
     {
-         std::for_each(v.begin(), v.end(), [&] (auto cmp) { this->operator()(cmp); });
+        std::for_each(v.begin(),
+                      v.end(),
+                      [&](auto cmp)
+                      {
+                          this->operator()(cmp);
+                      });
     }
 
 protected:
@@ -371,33 +379,41 @@ protected:
 
         // Check if extension is available.
         if (!path.has_extension())
-            throw validation_error{detail::to_string("The given filename ", path.string(),
-                                                            " has no extension. Expected one of the following valid"
-                                                            " extensions:", extensions, "!")};
+            throw validation_error{detail::to_string("The given filename ",
+                                                     path.string(),
+                                                     " has no extension. Expected"
+                                                     " one of the following valid extensions:",
+                                                     extensions,
+                                                     "!")};
 
-        // Drop the dot.
-        std::string drop_less_ext = path.extension().string().substr(1);
+        std::string file_path{path.filename().string()};
+
+        // Leading dot indicates a hidden file is not part of the extension.
+        if (file_path.front() == '.')
+            file_path.erase(0, 1);
+
+        // Store a string_view containing all extensions for a better error message.
+        std::string const all_extensions{file_path.substr(file_path.find(".") + 1)};
 
         // Compares the extensions in lower case.
-        auto case_insensitive_equal_to = [&] (std::string const & ext)
+        auto case_insensitive_ends_with = [&](std::string const & ext)
         {
-            return std::ranges::equal(ext, drop_less_ext, [] (char const chr1, char const chr2)
-                   {
-                       return std::tolower(chr1) == std::tolower(chr2);
-                   });
+            return case_insensitive_string_ends_with(file_path, ext);
         };
 
         // Check if requested extension is present.
-        if (std::ranges::find_if(extensions, case_insensitive_equal_to) == extensions.end())
+        if (std::ranges::find_if(extensions, case_insensitive_ends_with) == extensions.end())
         {
             throw validation_error{detail::to_string("Expected one of the following valid extensions: ",
-                                                             extensions, "! Got ", drop_less_ext, " instead!")};
+                                                     extensions,
+                                                     "! Got ",
+                                                     all_extensions,
+                                                     " instead!")};
         }
     }
 
     /*!\brief Checks if the given path is readable.
      * \param path The path to check.
-     * \returns `true` if readable, otherwise `false`.
      * \throws seqan3::validation_error if the path is not readable, or
      *         std::filesystem::filesystem_error on underlying OS API errors.
      */
@@ -407,9 +423,9 @@ protected:
         if (std::filesystem::is_directory(path))
         {
             std::error_code ec{};
-            std::filesystem::directory_iterator{path, ec};  // if directory iterator cannot be created, ec will be set.
+            std::filesystem::directory_iterator{path, ec}; // if directory iterator cannot be created, ec will be set.
             if (static_cast<bool>(ec))
-                throw validation_error{detail::to_string("Cannot read the directory ", path ,"!")};
+                throw validation_error{detail::to_string("Cannot read the directory ", path, "!")};
         }
         else
         {
@@ -425,7 +441,6 @@ protected:
 
     /*!\brief Checks if the given path is writable.
      * \param path The path to check.
-     * \returns `true` if writable, otherwise `false`.
      * \throws seqan3::validation_error if the file could not be opened for writing, or
      *         std::filesystem::filesystem_error on underlying OS API errors.
      */
@@ -450,7 +465,27 @@ protected:
         if (extensions.empty())
             return "";
         else
-            return detail::to_string(" Valid file extensions are: [", extensions | views::join(std::string{", "}), "].");
+            return detail::to_string(" Valid file extensions are: [",
+                                     extensions | views::join_with(std::string{", "}),
+                                     "].");
+    }
+
+    /*!\brief Helper function that checks if a string is a suffix of another string. Case insensitive.
+     * \param str The string to be searched.
+     * \param suffix The suffix to be searched for.
+     * \returns `true` if `suffix` is a suffix of `str`, otherwise `false`.
+     */
+    bool case_insensitive_string_ends_with(std::string_view str, std::string_view suffix) const
+    {
+        size_t const suffix_length{suffix.size()};
+        size_t const str_length{str.size()};
+        return suffix_length > str_length ? false
+                                          : std::ranges::equal(str.substr(str_length - suffix_length),
+                                                               suffix,
+                                                               [](char const chr1, char const chr2)
+                                                               {
+                                                                   return std::tolower(chr1) == std::tolower(chr2);
+                                                               });
     }
 
     //!\brief Stores the extensions.
@@ -478,12 +513,13 @@ protected:
  * \include test/snippet/argument_parser/validators_input_file_ext_from_file.cpp
  *
  * \note The validator works on every type that can be implicitly converted to std::filesystem::path.
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 template <typename file_t = void>
 class input_file_validator : public file_validator_base
 {
 public:
-
     static_assert(std::same_as<file_t, void> || detail::has_type_valid_formats<file_t>,
                   "Expected either a template type with a static member called valid_formats (a file type) or void.");
 
@@ -523,9 +559,7 @@ public:
      * static member `valid_formats`.
      */
     explicit input_file_validator(std::vector<std::string> extensions)
-    //!\cond
         requires std::same_as<file_t, void>
-    //!\endcond
         : file_validator_base{}
     {
         file_validator_base::extensions = std::move(extensions);
@@ -556,10 +590,12 @@ public:
             // Check extension.
             validate_filename(file);
         }
+        // LCOV_EXCL_START
         catch (std::filesystem::filesystem_error & ex)
         {
             std::throw_with_nested(validation_error{"Unhandled filesystem error!"});
         }
+        // LCOV_EXCL_STOP
         catch (...)
         {
             std::rethrow_exception(std::current_exception());
@@ -569,11 +605,19 @@ public:
     //!\brief Returns a message that can be appended to the (positional) options help page info.
     std::string get_help_page_message() const
     {
-        return "The input file must exist and read permissions must be granted." +
-               valid_extensions_help_page_message();
+        return "The input file must exist and read permissions must be granted." + valid_extensions_help_page_message();
     }
 };
 
+//!\brief Mode of an output file: Determines whether an existing file can be (silently) overwritten.
+enum class output_file_open_options
+{
+    //!\brief Allow to overwrite the output file
+    open_or_create,
+    //!\brief Forbid overwriting the output file
+    create_new
+};
+
 /*!\brief A validator that checks if a given path is a valid output file.
  * \ingroup argument_parser
  * \implements seqan3::validator
@@ -582,9 +626,13 @@ public:
  * \details
  *
  * On construction, the validator can receive a list (std::vector over std::string) of valid file extensions.
- * The class acts as a functor that throws a seqan3::validation_error exception whenever a given filename's
- * extension (sts::string) is not in the given list of valid file extensions, if the file already exist, or if the
- * parent path does not have the proper writer permissions.
+ * The class acts as a functor that throws a seqan3::validation_error exception whenever a given filename's extension
+ * (std::string) is not in the given list of valid file extensions, or if the parent path does not have the proper
+ * writer permissions.
+ * In addition, the validator receives a seqan3::output_file_open_options which allows you to specify what to do if your
+ * output file already exists. seqan3::output_file_open_options::create_new will throw a seqan3::validation_error
+ * exception if it already exists and seqan3::output_file_open_options::open_or_create will skip this check (that means
+ * you are allowed to overwrite the existing file).
  *
  * \include test/snippet/argument_parser/validators_output_file.cpp
  *
@@ -595,14 +643,15 @@ public:
  * \include test/snippet/argument_parser/validators_output_file_ext_from_file.cpp
  *
  * \note The validator works on every type that can be implicitly converted to std::filesystem::path.
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 template <typename file_t = void>
 class output_file_validator : public file_validator_base
 {
 public:
-
     static_assert(std::same_as<file_t, void> || detail::has_type_valid_formats<file_t>,
-                "Expected either a template type with a static member called valid_formats (a file type) or void.");
+                  "Expected either a template type with a static member called valid_formats (a file type) or void.");
 
     // Import from base class.
     using typename file_validator_base::option_value_type;
@@ -612,24 +661,25 @@ public:
      */
 
     //!\copydoc seqan3::input_file_validator::input_file_validator()
-    output_file_validator()
-    {
-        if constexpr (!std::same_as<file_t, void>)
-            file_validator_base::extensions = detail::valid_file_extensions<typename file_t::valid_formats>();
-    }
+    output_file_validator() : output_file_validator{output_file_open_options::create_new}
+    {}
 
-    output_file_validator(output_file_validator const &) = default;               //!< Defaulted.
-    output_file_validator(output_file_validator &&) = default;                    //!< Defaulted.
-    output_file_validator & operator=(output_file_validator const &) = default;   //!< Defaulted.
-    output_file_validator & operator=(output_file_validator &&) = default;        //!< Defaulted.
-    virtual ~output_file_validator() = default;                                   //!< Virtual Destructor.
+    output_file_validator(output_file_validator const &) = default;             //!< Defaulted.
+    output_file_validator(output_file_validator &&) = default;                  //!< Defaulted.
+    output_file_validator & operator=(output_file_validator const &) = default; //!< Defaulted.
+    output_file_validator & operator=(output_file_validator &&) = default;      //!< Defaulted.
+    virtual ~output_file_validator() = default;                                 //!< Virtual Destructor.
 
-    //!\copydoc seqan3::input_file_validator::input_file_validator(std::vector<std::string>)
-    explicit output_file_validator(std::vector<std::string> extensions)
-    //!\cond
-        requires std::same_as<file_t, void>
-    //!\endcond
-        : file_validator_base{}
+    /*!\brief Constructs from a given overwrite mode and a list of valid extensions.
+     * \param[in] mode A seqan3::output_file_open_options indicating whether the validator throws if a file already
+                       exists.
+     * \param[in] extensions The valid extensions to validate for. Defaults to
+     *                       seqan3::output_file_validator::default_extensions.
+     */
+    explicit output_file_validator(output_file_open_options const mode,
+                                   std::vector<std::string> extensions = default_extensions()) :
+        file_validator_base{},
+        mode{mode}
     {
         file_validator_base::extensions = std::move(extensions);
     }
@@ -638,6 +688,21 @@ public:
     using file_validator_base::file_validator_base;
     //!\}
 
+    /*!\brief The default extensions of `file_t`.
+     * \returns A list of default extensions for `file_t`, will be empty if `file_t` is `void`.
+     *
+     * \details
+     *
+     * If `file_t` does name a valid seqan3 file type that contains a static member `valid_formats` returns the
+     * extensions of that `file_t` type. Otherwise returns an empty list.
+     */
+    static std::vector<std::string> default_extensions()
+    {
+        if constexpr (!std::same_as<file_t, void>)
+            return detail::valid_file_extensions<typename file_t::valid_formats>();
+        return {};
+    }
+
     // Import the base::operator()
     using file_validator_base::operator();
 
@@ -650,7 +715,7 @@ public:
     {
         try
         {
-            if (std::filesystem::exists(file))
+            if ((mode == output_file_open_options::create_new) && std::filesystem::exists(file))
                 throw validation_error{detail::to_string("The file ", file, " already exists!")};
 
             // Check if file has any write permissions.
@@ -658,10 +723,12 @@ public:
 
             validate_filename(file);
         }
+        // LCOV_EXCL_START
         catch (std::filesystem::filesystem_error & ex)
         {
             std::throw_with_nested(validation_error{"Unhandled filesystem error!"});
         }
+        // LCOV_EXCL_STOP
         catch (...)
         {
             std::rethrow_exception(std::current_exception());
@@ -671,9 +738,16 @@ public:
     //!\brief Returns a message that can be appended to the (positional) options help page info.
     std::string get_help_page_message() const
     {
-        return "The output file must not exist already and write permissions must be granted." +
-               valid_extensions_help_page_message();
+        if (mode == output_file_open_options::open_or_create)
+            return "Write permissions must be granted." + valid_extensions_help_page_message();
+        else // mode == create_new
+            return "The output file must not exist already and write permissions must be granted."
+                 + valid_extensions_help_page_message();
     }
+
+private:
+    //!\brief Stores the current mode of whether it is valid to overwrite the output file.
+    output_file_open_options mode{output_file_open_options::create_new};
 };
 
 /*!\brief A validator that checks if a given path is a valid input directory.
@@ -689,6 +763,8 @@ public:
  * \include test/snippet/argument_parser/validators_input_directory.cpp
  *
  * \note The validator works on every type that can be implicitly converted to std::filesystem::path.
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 class input_directory_validator : public file_validator_base
 {
@@ -731,10 +807,12 @@ public:
             // Check if directory has any read permissions.
             validate_readability(dir);
         }
+        // LCOV_EXCL_START
         catch (std::filesystem::filesystem_error & ex)
         {
             std::throw_with_nested(validation_error{"Unhandled filesystem error!"});
         }
+        // LCOV_EXCL_STOP
         catch (...)
         {
             std::rethrow_exception(std::current_exception());
@@ -761,6 +839,8 @@ public:
  * \include test/snippet/argument_parser/validators_output_directory.cpp
  *
  * \note The validator works on every type that can be implicitly converted to std::filesystem::path.
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 class output_directory_validator : public file_validator_base
 {
@@ -813,10 +893,12 @@ public:
                 validate_writeability(dir / "dummy.txt");
             }
         }
+        // LCOV_EXCL_START
         catch (std::filesystem::filesystem_error & ex)
         {
             std::throw_with_nested(validation_error{"Unhandled filesystem error!"});
         }
+        // LCOV_EXCL_STOP
         catch (...)
         {
             std::rethrow_exception(std::current_exception());
@@ -837,7 +919,7 @@ public:
  * \details
  *
  * On construction, the validator must receive a pattern for a regular expression.
- * The pattern variable will be used for constructing an std::regex and the
+ * The pattern variable will be used for constructing a std::regex and the
  * validator will call std::regex_match on the command line argument.
  * Note: A regex_match will only return true if the strings matches the pattern
  * completely (in contrast to regex_search which also matches substrings).
@@ -846,6 +928,8 @@ public:
  * exception whenever string does not match the pattern.
  *
  * \include test/snippet/argument_parser/validators_4.cpp
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 class regex_validator
 {
@@ -856,8 +940,7 @@ public:
     /*!\brief Constructing from a vector.
      * \param[in] pattern_ The pattern to match.
      */
-    regex_validator(std::string const & pattern_) :
-        pattern{pattern_}
+    regex_validator(std::string const & pattern_) : pattern{pattern_}
     {}
 
     /*!\brief Tests whether cmp lies inside values.
@@ -878,12 +961,14 @@ public:
      * \throws seqan3::validation_error
      */
     template <std::ranges::forward_range range_type>
-    //!\cond
-        requires std::convertible_to<std::ranges::range_value_t<range_type>, option_value_type const &>
-    //!\endcond
+        requires std::convertible_to<std::ranges::range_reference_t<range_type>, option_value_type const &>
     void operator()(range_type const & v) const
     {
-         std::for_each(v.begin(), v.end(), [&] (auto cmp) { (*this)(cmp); });
+        for (auto && file_name : v)
+        {
+            // note: we explicitly copy/construct any reference type other than `std::string &`
+            (*this)(static_cast<option_value_type const &>(file_name));
+        }
     }
 
     //!\brief Returns a message that can be appended to the (positional) options help page info.
@@ -909,6 +994,8 @@ namespace detail
  *
  * The default validator is needed to make the validator parameter of
  * argument_parser::add_option and argument_parser::add_option optional.
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 template <typename option_value_t>
 struct default_validator
@@ -937,16 +1024,17 @@ struct default_validator
  * avoid unexpected behaviour and ensure that the seqan3::argument_parser::add_option
  * call is well-formed. (add_option(val, ...., validator) requires
  * that val is of same type as validator::option_value_type).
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 template <validator validator1_type, validator validator2_type>
-//!\cond
-    requires std::same_as<typename validator1_type::option_value_type, typename validator2_type::option_value_type>
-//!\endcond
+    requires std::common_with<typename validator1_type::option_value_type, typename validator2_type::option_value_type>
 class validator_chain_adaptor
 {
 public:
     //!\brief The underlying type in both validators.
-    using option_value_type = typename validator1_type::option_value_type;
+    using option_value_type =
+        std::common_type_t<typename validator1_type::option_value_type, typename validator2_type::option_value_type>;
 
     /*!\name Constructors, destructor and assignment
      * \{
@@ -962,7 +1050,8 @@ public:
      * \param[in] vali2_ Another validator to be chained to vali1_.
      */
     validator_chain_adaptor(validator1_type vali1_, validator2_type vali2_) :
-        vali1{std::move(vali1_)}, vali2{std::move(vali2_)}
+        vali1{std::move(vali1_)},
+        vali2{std::move(vali2_)}
     {}
 
     //!\brief The destructor.
@@ -978,9 +1067,7 @@ public:
      * the chained validators which may throw on input error.
      */
     template <typename cmp_type>
-    //!\cond
         requires std::invocable<validator1_type, cmp_type const> && std::invocable<validator2_type, cmp_type const>
-    //!\endcond
     void operator()(cmp_type const & cmp) const
     {
         vali1(cmp);
@@ -1028,16 +1115,15 @@ private:
  *
  * You can chain as many validators as you want which will be evaluated one after
  * the other from left to right (first to last).
+ *
+ * \remark For a complete overview, take a look at \ref argument_parser
  */
 template <validator validator1_type, validator validator2_type>
-//!\cond
-    requires std::same_as<typename std::remove_reference_t<validator1_type>::option_value_type,
-                       typename std::remove_reference_t<validator2_type>::option_value_type>
-//!\endcond
+    requires std::common_with<typename std::remove_reference_t<validator1_type>::option_value_type,
+                              typename std::remove_reference_t<validator2_type>::option_value_type>
 auto operator|(validator1_type && vali1, validator2_type && vali2)
 {
-    return detail::validator_chain_adaptor{std::forward<validator1_type>(vali1),
-                                           std::forward<validator2_type>(vali2)};
+    return detail::validator_chain_adaptor{std::forward<validator1_type>(vali1), std::forward<validator2_type>(vali2)};
 }
 
 } // namespace seqan3
